 /*
  * searchscreen.c - Dictionary Search Screen Functions
  * Copyright (c)2004 Pawel Szymczykowski <pawel@makenai.net>
  *
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * It is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Foobar; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#include "searchscreen.h"

extern GPDRAWSURFACE gpDraw[2];
extern BitFont * hankaku;
extern BitFont * zenkaku;
extern int currentSurface;
extern int mode;

int romaji2kana(char * term, char * buffer, char in)
{

	// Help me. I yearn to be implemented in QuickSearch Algoritm!
	
	int buf_pos = strlen(buffer);
	buffer[buf_pos] = in;
	buffer[++buf_pos] = '\0';
	
	for (int i=0;i<R2K_SIZE;i++)
	{
		if (strncmp(buffer, rom2kana_table[i].romaji, buf_pos) == 0)
		{
			if (strlen((char *) rom2kana_table[i].romaji) == buf_pos)
			{
				if (strlen(term) + strlen((char *) rom2kana_table[i].romaji) < 25) // Don't hard code me here, man.
					strcat(term, (const char *) rom2kana_table[i].kana);
				buffer[0] = '\0';
				return(1);
			} else {
				return(0);
			}
		}
	}
	buffer[buf_pos - 1] = '\0';
	return(0);
}

void searchScreen(char * term)
{
	int current_key = 0;
	char ch;

	int init = TRUE;

	char conv_buffer[10];

	term[0] = '\0';
	conv_buffer[0] = '\0';

	
	int banner_xpos = (LCD_WIDTH - banner_width) / 2;
	
	JConsole * console = new JConsole(banner_xpos + 2, 52, banner_xpos + banner_width - 2, 68, hankaku, zenkaku, 1); 
	
	currentSurface = !currentSurface;
	
	while (1)
	{
		if (!init)
		{
			ch = GpKeyGet();
			if (!GpKeyChanged()) continue;

			switch(ch)
			{
				case GPC_VK_UP:
					current_key = keymap[current_key].up;
				break;
				case GPC_VK_DOWN:
					current_key = keymap[current_key].down;
				break;
				case GPC_VK_LEFT:
					current_key = keymap[current_key].left;
				break;
				case GPC_VK_RIGHT:
					current_key = keymap[current_key].right;
				break;
				case GPC_VK_FR:
					if (mode == ETOJ)
					{
						if (strlen(term) >= 3) return;
					} else {
						if (strlen(term)) return;	
					}
				break;
				case GPC_VK_FL:
					term[0] = '\0';
					conv_buffer[0] = '\0';
					console->clear();
					if (mode == JTOE)
						mode = ETOJ;
					else
						mode = JTOE;
				break;
				case GPC_VK_FA:
					switch (keymap[current_key].value)
					{
						case EIWA_MODE:
							if (mode == JTOE)
							{
								term[0] = '\0';
								conv_buffer[0] = '\0';
								console->clear();
							}
							mode = ETOJ;
						break;

						case WAEI_MODE:
							if (mode == ETOJ)
							{
								term[0] = '\0';
								conv_buffer[0] = '\0';
								console->clear();
							}
							mode = JTOE;
						break;
						
						case LOOKUP:
							if (mode == ETOJ)
							{
								if (strlen(term) >= 3) return;
							} else {
								if (strlen(term)) return;	
							}
						break;
						
						default:
							console->clear();
							if (mode == ETOJ)
							{
								int cur_pos = strlen(term);
								if (cur_pos < 25)
								{
									term[cur_pos] = keymap[current_key].value;
									term[cur_pos + 1] = '\0';
								}
								console->print(term, 0x00);
							} 
							else if (mode == JTOE) 
							{
								romaji2kana(term, conv_buffer, 
									keymap[current_key].value);
								console->print(term, 0x00);
								console->print(conv_buffer, 0x00);
							}
						break;
					}
				break;
				case GPC_VK_FB:
					console->clear();
					if (mode == ETOJ)
					{
						int cur_pos = strlen(term);
						if (cur_pos > 0)
						{
							term[cur_pos - 1] = '\0';
						}
						console->print(term, 0x00);
					}
					else if (mode == JTOE)
					{
						if (strlen(conv_buffer))
						{
							conv_buffer[0] = '\0';
						}
						else 
						{
							int cur_pos = strlen(term);
							if (cur_pos > 1)
							{
								term[cur_pos - 2] = '\0';
							}
						}
						console->print(term, 0x00);
					}
				break;
			}
			console->scroll_dn();
		}
		init = FALSE;
		
		GpRectFill(NULL, &gpDraw[currentSurface], 0, 0, LCD_WIDTH, LCD_HEIGHT, 0xFF);
	
		GpRectDraw(&gpDraw[currentSurface], banner_xpos, 50, banner_xpos + banner_width, 69, 0x00);
		console->draw(&gpDraw[currentSurface]);
		
		GpRectFill(NULL, &gpDraw[currentSurface], 10 + keymap[current_key].xoff, 117 + keymap[current_key].yoff, 
			keymap[current_key].width, keymap[current_key].height, 0xB7);
		
		putxbm(&gpDraw[currentSurface], banner_xpos, 11, banner_bits, banner_width, banner_height, 0x00);

		if (mode == ETOJ)
		{
			putxbm(&gpDraw[currentSurface], (banner_xpos + banner_width + 1) - 
				etoj_width, 72, etoj_bits, etoj_width, etoj_height, 0x00);
		} else {
			putxbm(&gpDraw[currentSurface], (banner_xpos + banner_width + 1) - 
				jtoe_width, 72, jtoe_bits, jtoe_width, jtoe_height, 0x00);
		}

		putxbm(&gpDraw[currentSurface], 10, 117, keyboard_bits, keyboard_width, keyboard_height, 0x00);

		GpSurfaceFlip(&gpDraw[currentSurface]);
		currentSurface = !currentSurface;
	}
}
